/***************************************************************************
 *   Weather Module for Kadu communicator                                  *
 *   Copyright (C) 2004 Micha Maek (bysk)                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qmessagebox.h>

#include "weather_parser.h"

WeatherParser::WeatherParser(QString& page, ConfigFile* cf)
	: strPage(page),
	config(cf)
{
}

/**
	Formatowanie wyjciowej prognozy pogody
**/
QString WeatherParser::getData(ConfigFile* wConfig, Forecast& forecast)
{
	bool cs = wConfig->readBoolEntry("Default","CaseSensitive");
	
	int num_layouts = wConfig->readNumEntry("Default","Layouts");
	WDataValue dValue;
	
	QString locName;
	QString actualName;
	QValueList<WDataValue> valsList;
	QStringList nextDaysNamesList;
	QValueList<WDataValue> nextDaysValsList;
	
	int cursor = 0;
	for(int i=0; i<num_layouts; i++)
	{
		QString layoutSection = QString("Layout%1").arg(i+1);
		QString layoutName = wConfig->readEntry(layoutSection,"Name");
		int num_repeats = wConfig->readNumEntry(layoutSection,"Repeats");
		int num_values = wConfig->readNumEntry(layoutSection,"Values");
		
		for(int j=0; j<num_repeats; j++)
		{
			for(int k=0; k<num_values; k++)
			{
				QString valSection = QString("Layout%1Value%2").arg(i+1).arg(k+1);
				
				dValue.Name = wConfig->readEntry(valSection,"Name");
				dValue.Start = wConfig->readEntry(valSection,"Start");
				dValue.End = wConfig->readEntry(valSection,"End");
				dValue.Content = "";
				
				getDataValue(dValue,cursor, wConfig, cs);
				
				if(dValue.Content.isEmpty())
					continue;
				else if(dValue.Name == "Phantom")
					continue;
				
				if(layoutName == "Name")
					locName = dValue.Content;
				else if(layoutName == "Actual")
				{
					if(dValue.Name == "Name")
						actualName = dValue.Content;
					else
						valsList.push_back(dValue);
				}
				else if(layoutName == "DaysNames")
				{
					if(dValue.Name == "Actual")
						actualName = dValue.Content;
					else if(dValue.Name == "NextDays")
						nextDaysNamesList.push_back(dValue.Content);
				}
				else if(layoutName == "NextDays" && dValue.Name == "Name")
					nextDaysNamesList.push_back(dValue.Content);
				else
					nextDaysValsList.push_back(dValue);
			}
		}
	}
	
	QString Data("");
	
	ForecastPage fPage;
	QString icon, temperature;
	bool first_time = true;
	for(QValueList<WDataValue>::iterator it = valsList.begin(); it != valsList.end(); it++)
	{
		if((*it).Name == "Icon")
			icon = wConfig->readEntry("Icons",(*it).Content);
		else if ((*it).Name == "Temperature")
			temperature = (*it).Content;
		else
		{
			if(first_time)
				first_time = false;
			else
				Data += "<br>";
			Data += tr((*it).Name) + ": " + (*it).Content;
		}
	}
	
	fPage.dayName = actualName;
	fPage.icon = icon;
	fPage.temperature = temperature;
	fPage.forecast = Data;
	forecast.push_back(fPage);
	
	int num_vals = nextDaysValsList.count();
	int num_names = nextDaysNamesList.count();
	int T = num_vals/num_names;
	
	first_time = true;
	Data = "";
	for(int i=0; i<num_vals; i++)
	{
		WDataValue& val = nextDaysValsList[i];
	
		if(i%T == 0)
		{
			fPage.dayName = nextDaysNamesList[i/T];
			icon = "";
		}
		
		if(val.Name == "Icon")
			icon = wConfig->readEntry("Icons",val.Content);
		else if (val.Name == "Temperature")
			temperature = val.Content;
		else
		{
			if(first_time)
				first_time = false;
			else
				Data += "<br>";
				
			Data += tr(val.Name) + ": " + val.Content;
		}
		
		if(i%T == T-1)
		{
			fPage.icon = icon;
			fPage.temperature = temperature;
			fPage.forecast = Data;
			forecast.push_back(fPage);
			
			Data = "";
			first_time = true;
		}
	}

	return locName;
}

/**
	Wyawianie fragmentu ze strony
**/
void WeatherParser::getDataValue(WDataValue& wdata, int& cursor, ConfigFile* wConfig, bool CaseSensitive)
{
	long int start, end;
	long int startData;
	
	start = strPage.find(wdata.Start, cursor, CaseSensitive);
	if(start == -1)
		return;
	
	startData = start + wdata.Start.length();
	
	end = strPage.find(wdata.End, startData, CaseSensitive);
	if(end == -1)
		return;
	
	cursor = end;
	wdata.Content = tagClean(strPage.mid(startData, end-startData));
}

/**
	Zwraca wyszukane identyfikatory miast dla szukanego miasta
**/
void WeatherParser::getSearch(QStringList& names, QStringList& codes, ConfigFile* wConfig)
{
	bool CaseSensitive = wConfig->readBoolEntry("Default","CaseSensitive");

	long int start, separator, end, current;
	long int startData, sepData;
	bool idFirst;
	QString section, starttag, septag, endtag, first, second;
	
	int countResults = wConfig->readNumEntry("Name Search","SearchResults");
	
	for(int i=0; i<countResults; i++)
	{
		section = QString("SearchResult%1").arg(i+1);
		idFirst = wConfig->readBoolEntry(section,"IDFirst");
		starttag = wConfig->readEntry(section,"Start");
		septag = wConfig->readEntry(section,"Separator");
		endtag = wConfig->readEntry(section,"End");
	
		current = 0;
		do{
			start = strPage.find(starttag, current, CaseSensitive);	
			startData = start + starttag.length();
			separator = strPage.find(septag, startData, CaseSensitive);
			sepData = separator + septag.length();
			end = strPage.find(endtag, sepData, CaseSensitive);
			
			if(end != -1 && start != -1 && separator != -1)
			{
				first = strPage.mid(startData, separator-startData);
				second = strPage.mid(sepData, end-sepData);
				
				if(!first.isEmpty() && !second.isEmpty())
				{
					if(idFirst)
					{
						names.push_front(tagClean(second));
						codes.push_front(first);
					}
					else
					{
						names.push_front(tagClean(first));
						codes.push_front(second);
					}
				}
				current = end+endtag.length();
			}
		}while(end != -1 && start != -1 && separator != -1);
	}
}

/**
	Wyowienie szukanego ID miasta z adresu URL.
	(niektre serwisy przekierowywuj od razu na
	stron z prognoz, gdy znajd tylko jedno miasto)
**/
QString WeatherParser::getFastSearch(const QString& link, const ConfigFile& wConfig)
{
	QString starttag, endtag;
	long int start, end, startData;
	
	starttag = wConfig.readEntry("Name Search","FastSearch Start");
	endtag = wConfig.readEntry("Name Search","FastSearch End");
	
	start = link.find(starttag, 0, false);
	startData = start + starttag.length();
	
	if(endtag.isEmpty())
		end = link.length();
	else
		end = link.find(endtag, startData, false);
	
	if(start == -1 || end == -1)
		return "";
	else
		return link.mid(startData, end-startData);
}

/**
	Czyszczenie bufora z tagw HTML i zamienianie znakw specjalnych
**/
QString WeatherParser::tagClean(QString str)
{
	//str.replace("&deg;","");
	str.replace("&nbsp;"," ");
	
	int start, end;
	start = 0;
	do{
		start = str.find("<",start);
		end = str.find(">",start+1);
		
		if(start != -1 && end != -1)
			str.replace(start,end+1-start, " ");
			
	}while(start != -1 && end != -1);
	
	str.replace("\n"," ");
	str.replace("\r"," ");
	str.replace("  "," ");
	str.replace(" ,",",");
	str.replace(" .",".");
	str.replace(" :",":");
	str.replace(" / ","/");
	return str;
}
